@@ -37,7 +37,7 @@ aftp0 <- function(printlevel, ns, nn, id,
     ncov.save <- ncov
     ncov <- 0
     beta <- numeric(bdim)
-    for (i in 1:ns){
+    for (i in seq_len(ns)){
         beta[2 * i - 1] <- log(sum(Y[, 2] - Y[, 1]) / sum(Y[, 3]))
         beta[2*i] <- 0
     }
